#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

public enum DMAverageType
{
	SMA,
	EMA,
	HMA,
//	KAMA,
//	LSMA,
//	MAMA,
	SMMA,
	TEMA,
	TMA,
//	VMA,
	VWMA,
	WMA,
	ZEROLAGEMA
	
}

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.")]
    public class DMAChannel : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int			period		= 6;
		
		private	DataSeries		value1;
		private	DataSeries		value2;
		
		private DMAverageType dmaType = DMAverageType.SMA;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.RoyalBlue, 2), "HighPlot"));
			Add(new Plot(new Pen(Color.OrangeRed, 2), "LowPlot"));
			
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			Displacement        = 4;
			
			value1	= new DataSeries(this);
			value2	= new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
			
			value1.Set(SMA(High, 6)[0]); 
			value2.Set(SMA(Low, 6)[0]); 
			
	
            HighPlot.Set(value1[0]);
            LowPlot.Set(value2[0]);
			
			//value1.Set(CurrentBar == 0 ? Input[0] : Input[0] * (2.0 / (1 + Period)) + (1 - (2.0 / (1 + Period))) * Value[1]);
			
			
			//--------------------------------------------------------------------------------
			
			switch (dmaType)
			{
				case DMAverageType.SMA:
					value1.Set(SMA(High, 6)[0]); 
					value2.Set(SMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
				
				case DMAverageType.EMA:
					value1.Set(EMA(High, 6)[0]); 
					value2.Set(EMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
				case DMAverageType.HMA:
					value1.Set(HMA(High, 6)[0]); 
					value2.Set(HMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
					
//				case DMAverageType.KAMA:
//					raviLongMA=KAMA(lMA / 5,lMA, lMA * 3)[0];
//					raviLongMA=KAMA(sMA / 5,sMA, sMA * 3)[0];
//					diff=raviShortMA-raviLongMA;
//					toPlot=Math.Abs((diff/raviLongMA)*100);
//					
//					FallingPlot.Set(toPlot);
//					break;
//					
//					
//				case DMAverageType.KAMA:
//					value1.Set(SMMA(High, 6)[0]); 
//					value2.Set(SMMA(Low, 6)[0]); 
//			
//	
//            		HighPlot.Set(value1[0]);
//            		LowPlot.Set(value2[0]);
//					break;
//					
//											
//				case DMAverageType.LSMA:
//					break;

//				case DMAverageType.MAMA:
//					break;
					
				case DMAverageType.SMMA:
					value1.Set(SMMA(High, 6)[0]); 
					value2.Set(SMMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
				case DMAverageType.TEMA:
					value1.Set(TEMA(High, 6)[0]); 
					value2.Set(TEMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
				case DMAverageType.TMA:
					value1.Set(TMA(High, 6)[0]); 
					value2.Set(TMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
//				case DMAverageType.VMA:
//					value1.Set(VMA(High, High, 6)[0]); 
//					value2.Set(VMA(Low, Low, 6)[0]); 
//			
//	
//            		HighPlot.Set(value1[0]);
//            		LowPlot.Set(value2[0]);
//					break;
				
				case DMAverageType.VWMA:
					value1.Set(VWMA(High, 6)[0]); 
					value2.Set(VWMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
				case DMAverageType.WMA:
					value1.Set(WMA(High, 6)[0]); 
					value2.Set(WMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
					
					
				case DMAverageType.ZEROLAGEMA:
					value1.Set(ZeroLagEMA(High, 6)[0]); 
					value2.Set(ZeroLagEMA(Low, 6)[0]); 
			
	
            		HighPlot.Set(value1[0]);
            		LowPlot.Set(value2[0]);
					break;
							
			}
			
			
			
			
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries HighPlot
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowPlot
        {
            get { return Values[1]; }
        }
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		
		[Description("DMAType")]
        [Category("Parameters")]
        public DMAverageType DMAType
        {
            get { return dmaType; }
            set { dmaType = value; }
        }
		
		#endregion

    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DMAChannel[] cacheDMAChannel = null;

        private static DMAChannel checkDMAChannel = new DMAChannel();

        /// <summary>
        /// A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.
        /// </summary>
        /// <returns></returns>
        public DMAChannel DMAChannel(DMAverageType dMAType, int period)
        {
            return DMAChannel(Input, dMAType, period);
        }

        /// <summary>
        /// A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.
        /// </summary>
        /// <returns></returns>
        public DMAChannel DMAChannel(Data.IDataSeries input, DMAverageType dMAType, int period)
        {
            checkDMAChannel.DMAType = dMAType;
            dMAType = checkDMAChannel.DMAType;
            checkDMAChannel.Period = period;
            period = checkDMAChannel.Period;

            if (cacheDMAChannel != null)
                for (int idx = 0; idx < cacheDMAChannel.Length; idx++)
                    if (cacheDMAChannel[idx].DMAType == dMAType && cacheDMAChannel[idx].Period == period && cacheDMAChannel[idx].EqualsInput(input))
                        return cacheDMAChannel[idx];

            DMAChannel indicator = new DMAChannel();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.DMAType = dMAType;
            indicator.Period = period;
            indicator.SetUp();

            DMAChannel[] tmp = new DMAChannel[cacheDMAChannel == null ? 1 : cacheDMAChannel.Length + 1];
            if (cacheDMAChannel != null)
                cacheDMAChannel.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDMAChannel = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DMAChannel DMAChannel(DMAverageType dMAType, int period)
        {
            return _indicator.DMAChannel(Input, dMAType, period);
        }

        /// <summary>
        /// A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.
        /// </summary>
        /// <returns></returns>
        public Indicator.DMAChannel DMAChannel(Data.IDataSeries input, DMAverageType dMAType, int period)
        {
            return _indicator.DMAChannel(input, dMAType, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DMAChannel DMAChannel(DMAverageType dMAType, int period)
        {
            return _indicator.DMAChannel(Input, dMAType, period);
        }

        /// <summary>
        /// A 'displaced' moving average (DMA) is simply a normal moving average shifted to the right or left. A displaced moving average can be computed based upon a stock's closing price, high price and/or low price. A short-term DMA setup popularized by Tom Joseph (Trading Techniques, Inc.) involved 2 DMA averages. These 2 averages form a channel that make clear any trend change event, as well as the strength of a trend underway. The first average is a 6-day average of price highs displaced 4 places to the right. The second is a 6-day average of price lows displaced 4 places to the right. DMAs by themselves are not intended as buy and sell signals. Rather, DMAs are best applied as short-term 'triggers' and as short-term trend 'monitors' in conjunction with other, independent indicators.
        /// </summary>
        /// <returns></returns>
        public Indicator.DMAChannel DMAChannel(Data.IDataSeries input, DMAverageType dMAType, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DMAChannel(input, dMAType, period);
        }

    }
}
#endregion
